<?php
namespace Teknisa\CNC\Factories;

use Doctrine\ORM\EntityManager;

class VendasCNCFactory {

    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }
    
    public function getNoExistSells($params){
        $sql = "SELECT TO_CHAR(CO.DTVENDA, 'DD/MM/YYYY') DTVENDA, CO.CDFILIALCON, CO.CDNSU, CO.CDCUPOM, CO.VRVENDA, TR.NMTIPORECE FROM CONSMOVCAIXA CO
                JOIN TIPORECE TR
                 ON CO.CDTIPORECE = TR.CDTIPORECE
                  WHERE NOT EXISTS(
                    SELECT RE.* FROM RETVENDAADQ RE
                     WHERE CO.NRCONSMOVCAIXA = RE.NRCONSMOVCAIXA
                  )
                  AND DTVENDA BETWEEN TO_DATE(:DTVENDAINI, 'dd/mm/yy') AND TO_DATE(:DTVENDAFIN, 'dd/mm/yy')
                  AND ((VRVENDA BETWEEN :VRVENDAINI AND :VRVENDAFIN)OR ('T' = :P_TVRVENDA))
                  AND CDCUPOM LIKE :CDCUPOM
                  AND CDFILIALCON = :CDFILIAL
                ORDER BY CO.DTVENDA, CO.VRVENDA, CO.CDNSU";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function getConciliatorSellsFiltered($params, $query){
        $sql = $query; 
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function getConciliatorSells($params){
        $sql = "SELECT NVTOTAL, COUNT(*) NVCONCILIADO 
                  FROM (
                    SELECT COUNT(CO.NRCONSMOVCAIXA) OVER (PARTITION BY 1) NVTOTAL, R.NRRETVENDAADQ, R.NRPRODUTOADQ
                    FROM ESTABELECIMENTO E, CONSMOVCAIXA CO 
                    LEFT JOIN RETVENDAADQ R JOIN ADQUIRENTE AD 
                     ON AD.NRADQUIRENTE = R.NRADQUIRENTE
                     AND ((INSTR (:P_NRSEQADQ, '|'||TO_CHAR(AD.NRADQUIRENTE)||'|') > 0) OR ('T' = :P_TNRSEQADQ)) 
                       JOIN PRODUTOADQ PX 
                        ON PX.NRPRODUTOADQ = R.NRPRODUTOADQ
                        AND PX.NMPRODUTOADQ = :P_NMPRODADQ 
                        ON R.NRCONCLANC = CO.NRCONCLANC
                        AND R.IDCONCILIADO = 'C'
                           JOIN RETVENDAADQ RT JOIN ADQUIRENTE A 
                            ON A.NRADQUIRENTE = RT.NRADQUIRENTE
                            AND ((INSTR (:P_NRSEQADQ, '|'||TO_CHAR(A.NRADQUIRENTE)||'|') > 0) OR ('T' = :P_TNRSEQADQ))                                                                                                         
                              JOIN PRODUTOADQ PA 
                              ON PA.NRPRODUTOADQ = RT.NRPRODUTOADQ 
                              AND PA.NMPRODUTOADQ = :P_NMPRODADQ 
                       ON RT.NRCONCLANC = CO.NRCONCLANC
                    WHERE CO.DTVENDA BETWEEN TO_DATE(:DTVENDAINI, 'dd/mm/yy') AND TO_DATE(:DTVENDAFIN, 'dd/mm/yy')
                    AND :NRESTABELEC = E.NRESTABELEC)C JOIN PRODUTOADQ P 
                 ON C.NRPRODUTOADQ = P.NRPRODUTOADQ 
                 GROUP BY NVTOTAL"; 
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function getConciliatorSellsDetails($params){
        $sql = "SELECT R.IDCONCILIADO PRIMARY, '' PARENT, DECODE(R.IDCONCILIADO, 'C', 'Vendas Conciliadas', 'Vendas não Conciliadas') IDCONDICAO, NULL DTORDENACAO, NULL DTVENDA, '' CDNSU, '' CDCUPOM, 
                  SUM(R.VRBRUTVENDA) VRBRUTVENDA, NULL NRRETVENDAADQ, NULL NRADQUIRENTE, '' NMADQUIRENTE, PX.NRPRODUTOADQ, PX.NMPRODUTOADQ
                    FROM ESTABELECIMENTO E, CONSMOVCAIXA CO 
                    LEFT JOIN RETVENDAADQ R JOIN ADQUIRENTE AD 
                     ON AD.NRADQUIRENTE = R.NRADQUIRENTE
                     AND ((INSTR (:P_NRSEQADQ, '|'||TO_CHAR(AD.NRADQUIRENTE)||'|') > 0) OR ('T' = :P_TNRSEQADQ)) 
                       JOIN PRODUTOADQ PX 
                        ON PX.NRPRODUTOADQ = R.NRPRODUTOADQ
                        AND PX.NMPRODUTOADQ = :P_NMPRODADQ 
                        ON R.NRCONCLANC = CO.NRCONCLANC
                       AND CO.NRCONCLANC IS NOT NULL
                           JOIN RETVENDAADQ RT JOIN ADQUIRENTE A 
                            ON A.NRADQUIRENTE = RT.NRADQUIRENTE
                            AND ((INSTR (:P_NRSEQADQ, '|'||TO_CHAR(A.NRADQUIRENTE)||'|') > 0) OR ('T' = :P_TNRSEQADQ))                                                                                                         
                              JOIN PRODUTOADQ PA 
                              ON PA.NRPRODUTOADQ = RT.NRPRODUTOADQ 
                              AND PA.NMPRODUTOADQ = :P_NMPRODADQ 
                       ON RT.NRCONCLANC = CO.NRCONCLANC
                    WHERE CO.DTVENDA BETWEEN TO_DATE(:DTVENDAINI, 'dd/mm/yy') AND TO_DATE(:DTVENDAFIN, 'dd/mm/yy')
                    AND :NRESTABELEC = E.NRESTABELEC
                    GROUP BY R.IDCONCILIADO, PX.NMPRODUTOADQ
                  UNION
                    SELECT R.IDCONCILIADO || ' - ' || R.NRRETVENDAADQ PRIMARY, R.IDCONCILIADO PARENT, '' IDCONDICAO, R.DTVENDA DTORDENACAO, TO_CHAR(R.DTVENDA, 'DD/MM/YY') DTVENDA, 
                    R.CDNSU, CO.CDCUPOM, R.VRBRUTVENDA, R.NRRETVENDAADQ, R.NRADQUIRENTE, AD.NMADQUIRENTE, R.NRPRODUTOADQ, PX.NMPRODUTOADQ
                      FROM ESTABELECIMENTO E, CONSMOVCAIXA CO 
                      LEFT JOIN RETVENDAADQ R JOIN ADQUIRENTE AD 
                       ON AD.NRADQUIRENTE = R.NRADQUIRENTE
                       AND ((INSTR (:P_NRSEQADQ, '|'||TO_CHAR(AD.NRADQUIRENTE)||'|') > 0) OR ('T' = :P_TNRSEQADQ)) 
                         JOIN PRODUTOADQ PX 
                          ON PX.NRPRODUTOADQ = R.NRPRODUTOADQ
                          AND (INSTR (:P_NMPRODADQ, PX.NMPRODUTOADQ) > 0)
                          ON R.NRCONCLANC = CO.NRCONCLANC
                         AND CO.NRCONCLANC IS NOT NULL
                             JOIN RETVENDAADQ RT JOIN ADQUIRENTE A 
                              ON A.NRADQUIRENTE = RT.NRADQUIRENTE
                              AND ((INSTR (:P_NRSEQADQ, '|'||TO_CHAR(A.NRADQUIRENTE)||'|') > 0) OR ('T' = :P_TNRSEQADQ))                                                                                                         
                                JOIN PRODUTOADQ PA 
                                ON PA.NRPRODUTOADQ = RT.NRPRODUTOADQ 
                                AND (INSTR (:P_NMPRODADQ, PA.NMPRODUTOADQ) > 0)
                         ON RT.NRCONCLANC = CO.NRCONCLANC
                      WHERE CO.DTVENDA BETWEEN TO_DATE(:DTVENDAINI, 'dd/mm/yy') AND TO_DATE(:DTVENDAFIN, 'dd/mm/yy')
                      AND :NRESTABELEC = E.NRESTABELEC  
                      ORDER BY DTORDENACAO, PRIMARY"; 
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function getConciliatorSellsDetailsFiltered($params, $query){
        $sql = $query; 
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function getAnticipatedSellsFiltered($params, $query){
        $sql = $query; 
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function loadAnticipatedSellsDetailsFilter($params, $query){
        $sql = $query; 
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }


    public function getAnticipatedSells($params){
        $sql = "SELECT TO_DATE(TO_CHAR(R.DTCREDITO, 'DD/MM/YYYY'), 'DD/MM/YYYY') DTCREDITO, E.NRESTABELEC, E.NMESTABELEC, L.CDLOTEVENDA, A.NMADQUIRENTE,
                  SUM(R.VRBRUTVENDA) VRBRUTVENDA, SUM(R.VRLIQPGTO) VRLIQPGTO, R.VRTAXAANT,
                  A.NRADQUIRENTE, P.NRPRODUTOADQ, E.NRESTABELEC
                 FROM LOTEVENDA L, ADQUIRENTE A, RETFINANCADQ R, ESTABELECIMENTO E, PRODUTOADQ P
                WHERE R.NRLOTEVENDA = L.NRLOTEVENDA
                  AND R.NRADQUIRENTE = A.NRADQUIRENTE
                  AND R.NRPRODUTOADQ = P.NRPRODUTOADQ
                  AND R.NRESTABELEC = E.NRESTABELEC
                  AND L.NRADQUIRENTE = A.NRADQUIRENTE
                  AND TRUNC(R.DTCREDITO) BETWEEN :DTVENDAINI AND :DTVENDAFIN
                  AND ((INSTR ( :P_NRPRODUTOADQ, TO_CHAR(P.NRPRODUTOADQ)) <> 0) OR ('T' = :P_TNRPRODUTOADQ)) 
                  AND ((INSTR ( :P_NRADQUIRENTE, TO_CHAR(A.NRADQUIRENTE)) <> 0) OR ('T' = :P_TNRADQUIRENTE)) 
                  AND ((INSTR ( :P_NRESTABELEC,  TO_CHAR(E.NRESTABELEC))  <> 0) OR ('T' = :P_TNRESTABELEC)) 
                  AND ( :TALL_FIELDS = 'T' OR (INSTR(DTCREDITO || '_' || NMESTABELEC || '_' || NMADQUIRENTE || '_' || CDLOTEVENDA || '_' || VRBRUTVENDA || '_' || VRLIQPGTO || '_' || VRTAXAANT, :ALL_FIELDS) <> 0) )
                  AND R.VRTAXAANT IS NOT NULL
                  AND R.IDSTATUSVENDA = 'A'
                GROUP BY TO_DATE(TO_CHAR(R.DTCREDITO, 'DD/MM/YYYY'), 'DD/MM/YYYY'), E.NMESTABELEC, L.CDLOTEVENDA, A.NMADQUIRENTE, R.VRTAXAANT, A.NRADQUIRENTE, P.NRPRODUTOADQ, E.NRESTABELEC
                ORDER BY 1,2,3,5,4";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function loadAnticipatedSellsDetails($params){
        $sql = "SELECT TO_CHAR(S.DTVENDA, 'dd/mm/yy') as DTVENDA, S.CDNSU, C.CDCUPOM, S.VRBRUTVENDA, 
                  S.VRLIQPGTO, S.VRTAXAANTVENDA, S.VRTAXAADM, S.CDLOTEVENDA, S.NMADQUIRENTE, S.NMPRODUTOADQ
                  FROM (SELECT R.DTVENDA, R.CDNSU, R.VRBRUTVENDA, 
                    R.VRLIQPGTO, R.VRTAXAANTVENDA, R.VRTAXAADM, L.CDLOTEVENDA, A.NMADQUIRENTE, R.NRCONCLANC, P.NMPRODUTOADQ
                     FROM RETFINANCADQ R, PRODUTOADQ P, LOTEVENDA L, ADQUIRENTE A
                      WHERE R.NRLOTEVENDA = L.NRLOTEVENDA
                       AND R.NRADQUIRENTE    = A.NRADQUIRENTE
                       AND R.NRPRODUTOADQ    = P.NRPRODUTOADQ
                       AND ((INSTR (:P_NRPRODUTOADQ, '|' || TO_CHAR(P.NRPRODUTOADQ) ||'|' ) > 0) OR ('T' = :P_TNRPRODUTOADQ))
                       AND L.NRADQUIRENTE        = A.NRADQUIRENTE
                       AND A.NRADQUIRENTE    = :P_NRADQUIRENTE
                       AND L.CDLOTEVENDA     = :CDLOTEVENDA
                       AND R.VRTAXAANT = :VRTAXAANT
                       AND R.NRESTABELEC = :NRESTABELEC
                       AND P.NRPRODUTOADQ = R.NRPRODUTOADQ
                       AND L.IDTIPOLOTE = 'A')S 
                        LEFT JOIN CONSMOVCAIXA C
                         ON S.NRCONCLANC = C.NRCONCLANC
                         ORDER BY S.DTVENDA";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function loadFlagByname($params){
        $sql = "SELECT NRPRODUTOADQ 
                 FROM PRODUTOADQ 
                  WHERE NMPRODUTOADQ = :NMPRODUTOADQ";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
}
