<?php

namespace Teknisa\CNC\Services;

use Teknisa\CNC\Utilities;
use Teknisa\CNC\Factories;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class Cnc01400CancelamentosAjustes {

	protected $entityManager;
	protected $util;
	protected $cancelAdjustFactory;
	protected $errosPadroes = array(
        '001' => 'Total de crédito difere do total de débito.',
        '002' => 'Data fora do período contábil.',
        '003' => 'Partida %s - Conta contábil não cadastrada.',
        '004' => 'Partida %s - Histórico padrão não cadastrado.',
        '005' => 'Partida %s - Código comercial não cadastrado.',
        '006' => 'Partida %s - Valor da partida zerado.',
        '007' => 'Partida %s - Valor de rateio zerado.',
        '008' => 'Partida %s - Filial de rateio não cadastrada.',
        '009' => 'Partida %s - Centro de custo não cadastrado para filial informada.',
        '010' => 'Partida %s - Soma dos rateios difere do valor da partida.',
        '011' => 'Partida %s - Conta contábil sem código comercial informado.',
        '012' => 'Partida %s - Conta Contábil Gerencial sem rateio informado.',
        '013' => 'Partida %s - Centro de custo deve ser informado.',
        '014' => 'Produto %s sem conta contábil.',
        '015' => 'Lançamento fora do período contábil para o financeiro. (BCO, PAG e REC)',
        '016' => 'Lançamento fora do período contábil de documentos fiscais de entrada. (DFE)',
        '017' => 'Lançamento fora do período contábil para documentos fiscais de saída. (DFS)'
    );

	public function __construct(\Doctrine\ORM\EntityManager $entityManager, Utilities\UtilCNC $util, Factories\Cnc01400CancelamentosAjustesFactory $cancelAdjustFactory)  {
		$this->entityManager = $entityManager;
		$this->util          = $util;
		$this->cancelAdjustFactory  = $cancelAdjustFactory;
	}

	public function getCancelAdjustments($params) {
        $DTVENDAINI      = null;
        $DTVENDAFIN      = null;
        $DTCREDITOINI    = null;
        $DTCREDITOFIN    = null;
        $P_TNRESTABELEC  = null;
        $P_TNRADQUIRENTE = null;
        $P_TNRPRODUTOADQ = null;
        $P_TDTVENDA      = null;
        $P_TDTCREDITO    = null;
        
        $choosenOne = $this->util->getParamFromConditions($params, 'NRESTABELEC');
        if(!isset($choosenOne)){
            
            $P_NRESTABELEC = NULL;
            $P_NRADQUIRENTE = NULL;
            $P_NRPRODUTOADQ = NULL;
            
            for ($i=0; $i < sizeof($params); $i++) {
                if($params[$i]["columnName"] === 'LABELESTABELEC'){
                    $P_NRESTABELEC = $params[$i]['value'];
                }
            }
    
            for ($i=0; $i < sizeof($params); $i++) {
                if($params[$i]["columnName"] === 'NMADQUIRENTE'){
                    $P_NRADQUIRENTE = $params[$i]['value'];
                }
            }
    
            for ($i=0; $i < sizeof($params); $i++) {
                if($params[$i]["columnName"] === 'NMPRODUTOADQ'){
                    $P_NRPRODUTOADQ = $params[$i]['value'];
                }
            }
    
           for ($i=0; $i < sizeof($params); $i++) {
                if($params[$i]["columnName"] === 'DTVENDA'){
                    $DTVENDAINI = str_replace('%', '', $params[$i]['value'][0]);
                    $DTVENDAFIN = str_replace('%', '', $params[$i]['value'][1]);
                } if ($DTVENDAINI == "" && $DTVENDAFIN == ""){
                    $DTVENDAINI = null;
                    $DTVENDAFIN = null;
                }
            }
    
            for ($i=0; $i < sizeof($params); $i++) {
                if($params[$i]["columnName"] === 'DTCREDITO'){
                    $DTCREDITOINI = str_replace('%', '', $params[$i]['value'][0]);
                    $DTCREDITOFIN = str_replace('%', '', $params[$i]['value'][1]);
                } if ($DTCREDITOINI == "" && $DTCREDITOFIN == ""){
                    $DTCREDITOINI = null;
                    $DTCREDITOFIN = null;
                }
            }
            
            for ($i=0; $i < sizeof($params); $i++) {
                if($params[$i]["columnName"] === 'IDCONCILIADO'){
                    $P_IDCONCILIADO = $params[$i]['value'];
                }
            }
            
            if($P_NRESTABELEC === "Todos"){
                $P_TNRESTABELEC = 'T';
                $P_NRESTABELEC  = null; 
            }
    
            if($P_NRADQUIRENTE === "Todas"){
                $P_TNRADQUIRENTE = 'T';
                $P_NRADQUIRENTE  = null; 
            }
            
            if($P_NRPRODUTOADQ === "Todos"){
                $P_TNRPRODUTOADQ = 'T';
                $P_NRPRODUTOADQ  = null;
            }
    
            if($DTVENDAINI === null || $DTCREDITOINI === null){ 
                if($DTVENDAINI === null){
                    $P_TDTVENDA = 'T';
                }
                if($DTCREDITOINI === null){
                    $P_TDTCREDITO = 'T';
                }
                if($DTVENDAINI === null && $DTCREDITOINI === null){
                    throw new \Exception('DTVENDCREDERROR',1);
                }
            }
            if(is_array($P_NRADQUIRENTE) || is_array($P_NRPRODUTOADQ) || is_array($P_NRESTABELEC)){
                $query = "SELECT AJ.DTVENDA, AJ.DTCREDITO, AJ.NRRETFINANCADQ, AJ.NRLOTEVENDA, AJ.CDLOTEVENDA, AJ.NRESTABELEC, AJ.NRORG, AJ.CDESTABELEC, AJ.NMESTABELEC, AJ.NRADQUIRENTE, AJ.NMADQUIRENTE, AJ.NRPRODUTOADQ, AJ.NMPRODUTOADQ, AJ.CDNSU, AJ.DSAJUSTE,  AJ.CDEMPRESA, AJ.VRLIQPGTO, AJ.IDCONCILIADO, AJ.IDSITUACAO,  AJ.IDSTATUSVENDA, AJ.NRMOVBANC, AJ.CDFILIAL, AJ.CDREDUCONT, AJ.CDTPCTRECEB, AJ.CDTPDBDEBGER, AJ.CDTPCREDBANC, CR.IDCTBLCRBANC, DE.IDCTBLDBBANC, T.CDCLASSFINA FROM (SELECT TO_CHAR(R.DTVENDA, 'DD/MM/YYYY') DTVENDA, TO_CHAR(R.DTCREDITO, 'DD/MM/YYYY') DTCREDITO, R.NRRETFINANCADQ, L.NRLOTEVENDA, L.CDLOTEVENDA, E.CDESTABELEC, E.NRESTABELEC, E.NRORG, E.NMESTABELEC, A.NRADQUIRENTE, A.NMADQUIRENTE, P.NRPRODUTOADQ, P.NMPRODUTOADQ NMPRODUTOADQ, R.CDNSU, R.DSAJUSTE,  E.CDEMPRESA, R.VRLIQPGTO, DECODE(R.IDCONCILIADO, 'C', 'Sim', 'Não') IDCONCILIADO, R.IDSITUACAO,  R.IDSTATUSVENDA, R.NRMOVBANC, E.CDFILIAL, BUSCA_PARAMETRO_CNC(E.NRESTABELEC, A.NRADQUIRENTE, P.NRPRODUTOADQ, 'CONTCORR', 'N', NULL, NULL)CDREDUCONT, BUSCA_PARAMETRO_CNC(E.NRESTABELEC, A.NRADQUIRENTE, P.NRPRODUTOADQ, 'TIPOCONTA', 'N', NULL, NULL) CDTPCTRECEB, BUSCA_PARAMETRO_CNC(E.NRESTABELEC, A.NRADQUIRENTE, P.NRPRODUTOADQ, 'TIPODEB', 'N', NULL, NULL) CDTPDBDEBGER, BUSCA_PARAMETRO_CNC(E.NRESTABELEC, A.NRADQUIRENTE, P.NRPRODUTOADQ, 'TIPOCRED', 'N', NULL, NULL) CDTPCREDBANC FROM RETFINANCADQ R, LOTEVENDA L, ADQUIRENTE A, PRODUTOADQ P, ESTABELECIMENTO E WHERE R.NRLOTEVENDA = L.NRLOTEVENDA AND R.NRADQUIRENTE = A.NRADQUIRENTE AND R.NRPRODUTOADQ = P.NRPRODUTOADQ AND R.NRESTABELEC = E.NRESTABELEC AND L.NRADQUIRENTE = A.NRADQUIRENTE AND ((INSTR (:P_NRESTABELEC, '|'||TO_CHAR(E.NRESTABELEC)||'|') > 0) OR ('T' = :P_TNRESTABELEC)) AND ((INSTR (:P_NRADQUIRENTE,     '|'||TO_CHAR(A.NRADQUIRENTE)||'|') > 0) OR ('T' = :P_TNRADQUIRENTE)) AND ((INSTR (:P_NRPRODUTOADQ, '|'||TO_CHAR(P.NRPRODUTOADQ)||'|') > 0) OR ('T' = :P_TNRPRODUTOADQ)) AND ((:P_IDCONCILIADO = TO_CHAR(R.IDCONCILIADO)) OR ('T' = :P_IDCONCILIADO)) AND (((TRUNC(R.DTCREDITO) BETWEEN TO_DATE(:DTCREDITOINI, 'DD/MM/YY') AND TO_DATE(:DTCREDITOFIN, 'DD/MM/YY'))) OR ('T' = :P_TDTCREDITO)) AND (((TRUNC(R.DTVENDA) BETWEEN TO_DATE(:DTVENDAINI, 'DD/MM/YY') AND TO_DATE(:DTVENDAFIN, 'DD/MM/YY')))OR ('T' = :P_TDTVENDA)) AND R.IDSTATUSVENDA IN('C','E') ORDER BY R.DTCREDITO, E.NMESTABELEC, A.NMADQUIRENTE, R.VRLIQPGTO)AJ LEFT JOIN TIPOCTRECEBER T ON AJ.CDTPCTRECEB = T.CDTPCTRECEB LEFT JOIN TPCREDBANC CR ON AJ.CDTPCREDBANC = CR.CDTPCREDBANC LEFT JOIN TPDEBIBANC DE ON AJ.CDTPDBDEBGER = DE.CDTPDEBIBANC"; 
                if(is_array($P_NRADQUIRENTE)){
                    $paramsadquirirente = array(
                        ':P_NRADQUIRENTE' => $P_NRADQUIRENTE
                    );
                    foreach (array_keys($paramsadquirirente) as $index => $name) {
                        if(!empty($paramsadquirirente[$name])){
                            $query = $this->util->putArrayOnQuery($query, $name, $paramsadquirirente[$name]);
                            $paramsadquirirente[$name] = '';
                        }
                        else{
                            $paramsadquirirente[$name] = 'null';
                        }
                    }
                } else {
                    $paramsfilial = array(
                        ':P_NRADQUIRENTE' => 'null'
                    );
                     $query = preg_replace('/'.':P_NRADQUIRENTE'.'/', $paramsfilial[':P_NRADQUIRENTE'], $query, 4);
                } if(is_array($P_NRPRODUTOADQ)){
                    $paramsfilial = array(
                        ':P_NRPRODUTOADQ' => $P_NRPRODUTOADQ
                    );
                    foreach (array_keys($paramsfilial) as $index => $name) {
                        if(!empty($paramsfilial[$name])){
                            $query = $this->util->putArrayOnQuery($query, $name, $paramsfilial[$name]);
                            $paramsfilial[$name] = '';
                        }
                        else{
                            $paramsfilial[$name] = 'null';
                        }
                    }
                } else {
                    $paramsfilial = array(
                        ':P_NRPRODUTOADQ' => 'null'
                    );
                     $query = preg_replace('/'.':P_NRPRODUTOADQ'.'/', $paramsfilial[':P_NRPRODUTOADQ'], $query, 4);
                } if(is_array($P_NRESTABELEC)){
                    $paramsfilial = array(
                        ':P_NRESTABELEC' => $P_NRESTABELEC
                    );
                    foreach (array_keys($paramsfilial) as $index => $name) {
                        if(!empty($paramsfilial[$name])){
                            $query = $this->util->putArrayOnQuery($query, $name, $paramsfilial[$name]);
                            $paramsfilial[$name] = '';
                        }
                        else{
                            $paramsfilial[$name] = 'null';
                        }
                    }
                } 
                else {
                    $paramsfilial = array(
                        ':P_NRESTABELEC' => 'null'
                    );
                     $query = preg_replace('/'.':P_NRESTABELEC'.'/', $paramsfilial[':P_NRESTABELEC'], $query, 4);
                }
                    $parameters = array(
    	                ':P_TNRADQUIRENTE' => $P_TNRADQUIRENTE,
    	                ':P_TNRPRODUTOADQ' => $P_TNRPRODUTOADQ,
                        ':P_TNRESTABELEC'  => $P_TNRESTABELEC,
    	                ':DTVENDAINI'      => $DTVENDAINI,
    	                ':DTVENDAFIN'      => $DTVENDAFIN,
                        ':DTCREDITOINI'    => $DTCREDITOINI,
                        ':DTCREDITOFIN'    => $DTCREDITOFIN,
                        ':P_TDTVENDA'      => $P_TDTVENDA,
                        ':P_TDTCREDITO'    => $P_TDTCREDITO,
                        ':P_IDCONCILIADO'  => $P_IDCONCILIADO
                	); 
                $CancelAjust = $this->cancelAdjustFactory->getCancelAdjustmentsFiltered($parameters, $query);
    			return $CancelAjust;
    		} else{
    			$parameters = array(
                    ':P_NRADQUIRENTE'   => $P_NRADQUIRENTE,
                    ':P_TNRADQUIRENTE'  => $P_TNRADQUIRENTE,
                    ':P_NRPRODUTOADQ'   => $P_NRPRODUTOADQ,
                    ':P_TNRPRODUTOADQ'  => $P_TNRPRODUTOADQ,
                    ':P_NRESTABELEC'    => $P_NRESTABELEC,
                    ':P_TNRESTABELEC'   => $P_TNRESTABELEC,
                    ':DTVENDAINI'       => $DTVENDAINI,
                    ':DTVENDAFIN'       => $DTVENDAFIN,
                    ':DTCREDITOINI'     => $DTCREDITOINI,
                    ':DTCREDITOFIN'     => $DTCREDITOFIN,
                    ':P_TDTVENDA'       => $P_TDTVENDA,
                    ':P_TDTCREDITO'     => $P_TDTCREDITO,
                    ':P_IDCONCILIADO'   => $P_IDCONCILIADO
            	);
                $CancelamentoAjuste = $this->cancelAdjustFactory->getCancelAdjustments($parameters);
                return $CancelamentoAjuste;
            } 
        }
        throw new \Exception('DTVENDCREDERROR',1);
	}
	
	public function changeSituationAdjustments($params) {
        $connection = $this->entityManager->getConnection();
        try{
            
            $NRESTABELEC    = $params[0]["NRESTABELEC"];
    	    $NRADQUIRENTE   = $params[0]["NRADQUIRENTE"];
    	    $NMADQUIRENTE   = $params[0]["NMADQUIRENTE"];
    	    $NRPRODUTOADQ   = $params[0]["NRPRODUTOADQ"];
    	    $NRRETFINANCADQ = $params[0]["NRRETFINANCADQ"];
            $IDCONCILIADO   = $params[0]["IDCONCILIADO"];
            $IDSITUACAO     = $params[0]["IDSITUACAO"];
            $IDDEBCRED      = $params[0]["IDDEBCRED"];
            $DESCMOVBANC    = $params[0]["DESCMOVBANC"];
            $DESCMOV        = $params[0]["DESCMOV"];
            $CDNSU          = $params[0]["CDNSU"];
            $EST_NRORG      = $params[0]["NRORG"];
            $DTVENDA        = $params[0]["DTVENDA"];
            $DTCREDITO      = $params[0]["DTCREDITO"];
            $CDFILIAL       = $params[0]["CDFILIAL"];
            $CDCLASSFINA    = $params[0]["CDCLASSFINA"];
            $CDTPCREDBANC   = $params[0]["CDTPCREDBANC"];
            $VRLIQPGTO      = $params[0]["VRLIQPGTO"];
            $CDEMPRESA      = $params[0]["CDEMPRESA"];
            $IDCTBLCRBANC   = $params[0]["IDCTBLCRBANC"];
            $IDCTBLDBBANC   = $params[0]["IDCTBLDBBANC"];
            $NRMOVBANC      = $params[0]["NRMOVBANC"];
            $CDREDUCONT     = $params[0]["CDREDUCONT"];
            $CDTPDBDEBGER   = $params[0]["CDTPDBDEBGER"];
            $IDSTATUSVENDA  = $params[0]["IDSTATUSVENDA"];
            
            $infoFinancAdq   = $this->cancelAdjustFactory->checkFinancAdq(array( 
                ':NRADQUIRENTE'     =>      $NRADQUIRENTE
            ));
            
            $IDGERAFINANC = $infoFinancAdq['IDGERAFINANC'];
            $CDTPDBDEBGER = $infoFinancAdq['CDTPDBDEBGER'];
            $CDTPCREDBANCGER = $infoFinancAdq['CDTPCREDBANCGER'];
            
            $infoRetFinanc = $this->cancelAdjustFactory->checkInfoRetfinanc(array(
                ':NRRETFINANCADQ'   =>      $NRRETFINANCADQ
            ));
            $IDCONCILIADO  = $infoRetFinanc['IDCONCILIADO'];
            $IDSITUACAO    = $infoRetFinanc['IDSITUACAO'];
            $IDSTATUSVENDA = $infoRetFinanc['IDSTATUSVENDA'];

            if($VRLIQPGTO < 0){
                if($IDSTATUSVENDA == 'C'){
                    $CDCLASSFINA = $this->cancelAdjustFactory->checkCDCLASSFINACancelamento(array(
                        ':P_NRESTABELEC'    =>  $NRESTABELEC,
                        ':P_NRADQUIRENTE'   =>  $NRADQUIRENTE,
                        ':P_NRPRODUTOADQ'   =>  $NRPRODUTOADQ,
                        ':P_TIPOPARAM'      =>  'TIPODEB',
                        ':P_GRAVAERRO'      =>  NULL,
                        ':P_DTINICIAL'      =>  NULL,
                        ':P_DTFINAL'        =>  NULL
                    ))['CDCLASSFINA'];
                }else{
                    $CDCLASSFINA = $this->cancelAdjustFactory->checkCDCLASSFINADebito(array(
                        ':NRRETFINANCADQ' => $NRRETFINANCADQ
                    ))['CDCLASSFINA'];
                    
                    $IDCTBLDBBANC = $this->cancelAdjustFactory->checkCDCLASSFINADebito(array(
                        ':NRRETFINANCADQ' => $NRRETFINANCADQ
                    ))['IDCTBLDBBANC'];
                }
            }else{
                $CDCLASSFINA = $this->cancelAdjustFactory->checkCDCLASSFINACredito(array(
                    ':NRRETFINANCADQ' => $NRRETFINANCADQ
                ))['CDCLASSFINA'];    
                
                $IDCTBLCRBANC = $this->cancelAdjustFactory->checkCDCLASSFINACredito(array(
                    ':NRRETFINANCADQ' => $NRRETFINANCADQ
                ))['IDCTBLCRBANC'];    
            }
            
		    switch ($IDSTATUSVENDA) {
		        case "P":
                    $DESCMOV = 'TAXAS DE ADQUIRENTE';
                    $DESCMOVBANC = 'AjusteNegativoAdquirente';
                    $IDDEBCRED = 'D';
                    break;
                case "C":
                    $DESCMOV = 'CANCELAMENTO DE VENDA';
                    $DESCMOVBANC = 'CancVenda';
                    $IDDEBCRED = 'D';
                    break;
                default:
                    $DESCMOV = 'AJUSTE POSITIVO DE ADQUIRENTE';
                    $DESCMOVBANC = 'AjustePositivoAdquirente';
                    $IDDEBCRED = 'C';
            }
            
            $paramsCnc = $this->cancelAdjustFactory->searchParamsCnc(array(
                ':NRESTABELEC'      => $NRESTABELEC,
                ':NRADQUIRENTE'     => $NRADQUIRENTE,
                ':NRPRODUTOADQ'     => $NRPRODUTOADQ
                ));
            
            $CDREDUCONT = $paramsCnc['CDREDUCONT'];
            $CDTPCREDBANC = $paramsCnc['CDTPCREDBANC']; 
                
            if($IDGERAFINANC == 'S'){
        	    if($IDCONCILIADO == "Sim"){
        	        $IDCONCILIADO = 'P';
        	        $IDSITUACAO   = 'N';
        	        $paramsdateAdj = array(
                        ':NRRETFINANCADQ'  => $NRRETFINANCADQ
                    );
                    $paramsTpBanc = array(
                        ':NRADQUIRENTE' => $NRADQUIRENTE
                    );
                    
                    $verifyCdTpBanc = $this->cancelAdjustFactory->getCdTpBancs($paramsTpBanc);
                    $paramIdDebBanc = array(
                        ':CDTPDEBIBANC' => $verifyCdTpBanc[0]['CDTPDBDEBGER']
                    );
                    
                    $paramIdCredBanc = array(
                        ':CDTPCREDBANC' => $verifyCdTpBanc[0]['CDTPCREDBANCGER']      
                    );
    
                    $verifyDebBanc = $this->cancelAdjustFactory->getIdDebBanc($paramIdDebBanc)[0]["IDCTBLDBBANC"];
                    $verifyCredBanc = $this->cancelAdjustFactory->getIdCredBanc($paramIdCredBanc)[0]["IDCTBLCRBANC"];
                    if($IDDEBCRED == 'D'){
                        if(($verifyDebBanc == 'S')){
                            $resultdateAdj = $this->cancelAdjustFactory->validateAdjustmentDate($paramsdateAdj);
                            if (count($resultdateAdj) == 0){
                                    $message = 'Operação Bloqueada! '.$this->errosPadroes["002"];
                                    throw new \Exception('ERR/'.$message, 10);
                            }
                        }
                        
                        $paramsMovBanc = array(
                            ':NRMOVBANC'  => $NRMOVBANC,
                            ':CDREDUCONT' => $CDREDUCONT
                        );

        	            $this->refreshCashFlow($CDEMPRESA, $CDFILIAL, $DTCREDITO, $CDCLASSFINA, $VRLIQPGTO, 'C');
                        $this->refreshBankBalance($CDREDUCONT, $DTCREDITO, $VRLIQPGTO, 'C');
                        $this->cancelAdjustFactory->lancDeb($paramsMovBanc);
                        $this->cancelAdjustFactory->movDebBanc($paramsMovBanc);
        	        
                    }else{
                        if(($verifyCredBanc == 'S')){
                            $resultdateAdj = $this->cancelAdjustFactory->validateAdjustmentDate($paramsdateAdj);
                            if (count($resultdateAdj) == 0){
                                    $message = 'Operação Bloqueada! '.$this->errosPadroes["002"];
                                    throw new \Exception('ERR/'.$message, 10);
                            }
                        }
                        $paramsMovBanc = array(
                            ':NRMOVBANC'  => $NRMOVBANC,
                            ':CDREDUCONT' => $CDREDUCONT
                        );

        	            $this->refreshCashFlow($CDEMPRESA, $CDFILIAL, $DTCREDITO, $CDCLASSFINA, $VRLIQPGTO, 'D');
                        $this->refreshBankBalance($CDREDUCONT, $DTCREDITO, $VRLIQPGTO, 'D');
                        $this->cancelAdjustFactory->lancCred($paramsMovBanc);
                        $this->cancelAdjustFactory->movCredBanc($paramsMovBanc);
            	         
                    }
        	        $this->cancelAdjustFactory->clearBankMoveRet($paramsdateAdj);
                    $this->cancelAdjustFactory->clearBankMove($paramsMovBanc);
                } else{
        	        $IDCONCILIADO = 'C';
        	        $IDSITUACAO   = 'C';
        	        $paramsBankadq = array(
                        ':NRESTABELEC'  => $NRESTABELEC,
                        ':NRADQUIRENTE' => $NRADQUIRENTE,
                        ':NRPRODUTOADQ' => $NRPRODUTOADQ
                    );
        	        $resultBank = $this->cancelAdjustFactory->validateBankFinestadq($paramsBankadq);
                    $resultBank[0]["CDREDUCONT"]      = isset($resultBank[0]["CDREDUCONT"]) ? $resultBank[0]["CDREDUCONT"] : $CDREDUCONT;
                    $resultBank[0]["CDTPDBDEBGER"]    = isset($resultBank[0]["CDTPDBDEBGER"]) ? $resultBank[0]["CDTPDBDEBGER"] : $CDTPDBDEBGER;
                    $resultBank[0]["CDTPCREDBANC"]    = isset($resultBank[0]["CDTPCREDBANC"]) ? $resultBank[0]["CDTPCREDBANC"] : $CDTPCREDBANC;
                    $resultBank[0]["IDCTBLCRBANC"]    = isset($resultBank[0]["IDCTBLCRBANC"]) ? $resultBank[0]["IDCTBLCRBANC"] : $IDCTBLCRBANC;
                    $resultBank[0]["IDCTBLDBBANC"]    = isset($resultBank[0]["IDCTBLDBBANC"]) ? $resultBank[0]["IDCTBLDBBANC"] : $IDCTBLDBBANC;
                    $resultBank[0]["CDTPDEBIBANC"]    = isset($resultBank[0]["CDTPDEBIBANC"]);
                    $resultBank[0]["CDTPCREDBANCGER"] = isset($resultBank[0]["CDTPCREDBANCGER"]) ? $resultBank[0]["CDTPCREDBANCGER"] : $CDTPCREDBANCGER;
                    
                    if ($resultBank[0]["CDREDUCONT"] == null)
                            throw new \Exception('ERR/'.'Operação Bloqueada! Não foi possível realizar a conciliação devido à falta de parametrização da Conta Corrente para a Adquirente '.$NMADQUIRENTE.'.', 1);
                    if($VRLIQPGTO < 0){
                        if ($resultBank[0]["CDTPDBDEBGER"] == null)
                            throw new \Exception('ERR/'.'Operação Bloqueada! Não foi possível realizar a conciliação devido à falta de Tipo de Débito (Genérico) parametrizado para a Adquirente '.$NMADQUIRENTE.'.', 2);
                        if ($resultBank[0]["CDTPDEBIBANC"] == null && $IDSTATUSVENDA == 'C')
                            throw new \Exception('ERR/'.'Operação Bloqueada! Não foi possível realizar a conciliação devido à falta de Tipo de Débito (Cancelamento) parametrizado para a Adquirente '.$NMADQUIRENTE.'.', 2);
                    }else{
                        if ($resultBank[0]["CDTPCREDBANCGER"] == null)
                            throw new \Exception('ERR/'.'Operação Bloqueada! Não foi possível realizar a conciliação devido à falta de Tipo de Crédito (Genérico) parametrizado para a Adquirente '.$NMADQUIRENTE.'.', 2);
                        if ($resultBank[0]["CDTPCREDBANC"] == null)
                            throw new \Exception('ERR/'.'Operação Bloqueada! Não foi possível realizar a conciliação devido à falta de Tipo de Crédito parametrizado para a Adquirente'.$NMADQUIRENTE.'.', 8);
                    }
                    $paramsdateAdj = array(
                        ':NRRETFINANCADQ'  => $NRRETFINANCADQ
                    );
                    $paramsTpBanc = array(
                        ':NRADQUIRENTE' => $NRADQUIRENTE
                    );
                    $verifyCdTpBanc = $this->cancelAdjustFactory->getCdTpBancs($paramsTpBanc);
                    $paramIdDebBanc = array(
                        ':CDTPDEBIBANC' => $verifyCdTpBanc[0]['CDTPDBDEBGER']
                    );
                    $paramIdCredBanc = array(
                        ':CDTPCREDBANC' => $verifyCdTpBanc[0]['CDTPCREDBANCGER']      
                    );
                    $verifyDebBanc = $this->cancelAdjustFactory->getIdDebBanc($paramIdDebBanc)[0]["IDCTBLDBBANC"];
                    $verifyCredBanc = $this->cancelAdjustFactory->getIdCredBanc($paramIdCredBanc)[0]["IDCTBLCRBANC"];
                    
                    if($IDDEBCRED == 'D'){
                        if(($verifyDebBanc == 'S')){
                            $resultdateAdj = $this->cancelAdjustFactory->validateAdjustmentDate($paramsdateAdj);
                            if (count($resultdateAdj) == 0){
                                    $message = 'Operação Bloqueada! '.$this->errosPadroes["002"];
                                    throw new \Exception('ERR/'.$message, 3);
                            }
                        }
                    }else{                        
                        if(($verifyCredBanc == 'S')){
                            $resultdateAdj = $this->cancelAdjustFactory->validateAdjustmentDate($paramsdateAdj);
                            if (count($resultdateAdj) == 0){
                                    $message = 'Operação Bloqueada! '.$this->errosPadroes["002"];
                                    throw new \Exception('ERR/'.$message, 10);
                            }
                        }
                    }
                    if($IDSTATUSVENDA == 'C' || $CDNSU != null){
                        $paramscontNtFsc = array(
                            ':CDNSU'         => $CDNSU,
                            ':DTVENDA'       => $DTVENDA,
                            ':NRADQUIRENTE'  => $NRADQUIRENTE
                        );
                        $resultcontNtFsc = $this->cancelAdjustFactory->validateContNotaFiscal($paramscontNtFsc);
                        if(isset($resultcontNtFsc[0]["CDFILIAL"]) || isset($resultcontNtFsc[0]["NRLANCTONF"])){
                            if($resultcontNtFsc[0]["CDFILIAL"] != null || $resultcontNtFsc[0]["NRLANCTONF"] != null){
                                $paramsdelContNtFsc = array(
                                    ':CDFILIAL'   => $resultcontNtFsc[0]["CDFILIAL"],
                                    ':NRLANCTONF' => $resultcontNtFsc[0]["NRLANCTONF"]
                                );
                                $this->cancelAdjustFactory->deleteContNotaFiscal($paramsdelContNtFsc);
                            }
                        }
                    }
                    $NRMOVBANC = $this->util->newCodeNT('MOVBANCARIO'.$resultBank[0]["CDREDUCONT"], '10');
                    $paramsbankmove = array(
                        ':CDREDUCONT'  => $resultBank[0]["CDREDUCONT"],
                        ':NRMOVBANC'   => $NRMOVBANC,
                        ':DTCREDITO'   => $DTCREDITO,
                        ':IDDEBCRED'   => $IDDEBCRED,
                        ':DESCMOVBANC' => $DESCMOVBANC,
                        ':VRLIQPGTO'   => $VRLIQPGTO,
                        ':EST_NRORG'   => $EST_NRORG,
                        ':CDNSU'       => $CDNSU
                    );
                    $this->cancelAdjustFactory->changeBankMove($paramsbankmove);
                    $params = array(
                        ':NRRETFINANCADQ' => $NRRETFINANCADQ,
                        ':CDREDUCONT'     => $resultBank[0]["CDREDUCONT"],
                        ':NRMOVBANC'      => $NRMOVBANC
                    );
                    
                    $this->cancelAdjustFactory->changeMovBancAdjustments($params);
                    
                    $paramsBankadq = array(
                        ':NRADQUIRENTE' => $NRADQUIRENTE
                    );
                    $setClient = $this->cancelAdjustFactory->getClient($paramsBankadq);
                    $setClient[0]["CDCLIENTE"] = isset($setClient[0]["CDCLIENTE"]) ? $setClient[0]["CDCLIENTE"] : null;
                    $CDCLIENTE = $setClient[0]["CDCLIENTE"];
                    
                    if($VRLIQPGTO < 0){
                       $this->refreshCashFlow($CDEMPRESA, $CDFILIAL, $DTCREDITO, $CDCLASSFINA, $VRLIQPGTO, 'D');
                       $this->refreshBankBalance($resultBank[0]["CDREDUCONT"], $DTCREDITO, $VRLIQPGTO, 'D');
                       $NRMOVIDEBI = $this->util->newCodeNT('MOVDEBBANC'.$resultBank[0]["CDREDUCONT"], '10');
                       $paramsdebbanc = array(
                        ':CDREDUCONT'  => $resultBank[0]["CDREDUCONT"],
                        ':NRMOVIDEBI'  => $NRMOVIDEBI,
                        ':CDCLASSFINA' => $CDCLASSFINA,
                        ':CDEMPRESA'   => $CDEMPRESA,
                        ':CDFILIAL'    => $CDFILIAL,
                        ':NRMOVBANC'   => $NRMOVBANC,
                        ':CDTPDEBIBANC'=> $resultBank[0]["CDTPDBDEBGER"],
                        ':DTCREDITO'   => $DTCREDITO,
                        ':DESCMOV'     => $DESCMOV,
                        ':VRLIQPGTO'   => $VRLIQPGTO,
                        ':EST_NRORG'   => $EST_NRORG,
                        ':CDCLIENTE'   => $CDCLIENTE,
                        ':CDECTBMDEB'  => $CDEMPRESA
                        
                       );
                       $this->cancelAdjustFactory->changeMoveDebBanc($paramsdebbanc);
                        if($resultBank[0]["IDCTBLDBBANC"] == 'S'){
                            $return = $this->ctbBcoLanc($resultBank[0]["CDREDUCONT"], $NRMOVIDEBI, 'D');
                            $NRLANCCTBTEMP = $return["P_NRLANCCTB"];
                            $paramserrors = array(
                                ':CDEMPRESA'      => $CDEMPRESA,
                                ':NRLANCCTBTEMP'  => $NRLANCCTBTEMP
                            );
                            $errors = $this->cancelAdjustFactory->getErrors($paramserrors);
                            if($errors[0]["QT"] != 0){
                                $error = $this->cancelAdjustFactory->getError($paramserrors);
                                $message = "";
                                foreach ($error as $key => $value) {
                                    if($key > 0){
                                        $message .= '|'.$this->errosPadroes[$error[$key]["CDERROTEMP"]];
                                    } else {
                                        $message .= $this->errosPadroes[$error[$key]["CDERROTEMP"]];
                                    }
                                }
                                throw new \Exception('CTB/'.$CDEMPRESA.'|'.$NRLANCCTBTEMP, 16);
                            }
                        }
                        
                    } else{
                       $this->refreshCashFlow($CDEMPRESA, $CDFILIAL, $DTCREDITO, $CDCLASSFINA, $VRLIQPGTO, 'C');
                       $this->refreshBankBalance($resultBank[0]["CDREDUCONT"], $DTCREDITO, $VRLIQPGTO, 'C');
                       $NRMOVICRED = $this->util->newCodeNT('MOVCREDBANC'.$CDREDUCONT, '10');
                       $paramscredbanc = array(
                        ':CDREDUCONT'  => $resultBank[0]["CDREDUCONT"],
                        ':NRMOVICRED'  => $NRMOVICRED,
                        ':CDCLASSFINA' => $CDCLASSFINA,
                        ':CDEMPRESA'   => $CDEMPRESA,
                        ':CDFILIAL'    => $CDFILIAL,
                        ':NRMOVBANC'   => $NRMOVBANC,
                        ':CDTPCREDBANC'=> $resultBank[0]["CDTPCREDBANC"],
                        ':DTCREDITO'   => $DTCREDITO,
                        ':DESCMOV'     => $DESCMOV,
                        ':VRLIQPGTO'   => $VRLIQPGTO,
                        ':EST_NRORG'   => $EST_NRORG,
                        ':CDCLIENTE'   => $CDCLIENTE,
                        ':CDFILIAL'    => $CDFILIAL,
                        ':CDECTBMCRED' => $CDEMPRESA
                        );
                       $this->cancelAdjustFactory->changeMoveCredBanc($paramscredbanc);
                        if($resultBank[0]["IDCTBLCRBANC"] == 'S'){
                            $return = $this->ctbBcoLanc($resultBank[0]["CDREDUCONT"], $NRMOVICRED, 'C');
                            $NRLANCCTBTEMP = $return["P_NRLANCCTB"];
                            $paramserrors = array(
                                ':CDEMPRESA'      => $CDEMPRESA,
                                ':NRLANCCTBTEMP'  => $NRLANCCTBTEMP
                            );
                            $errors = $this->cancelAdjustFactory->getErrors($paramserrors);
                            if($errors[0]["QT"] != 0){
                                throw new \Exception('CTB/'.$CDEMPRESA.'|'.$NRLANCCTBTEMP, 9);
                            }
                        } 
                    }
            	}
            } else{
                if($IDCONCILIADO == "Sim"){
                    $IDCONCILIADO = 'P';
        	        $IDSITUACAO   = 'N';
                } else{
                    $IDCONCILIADO = 'C';
        	        $IDSITUACAO   = 'C'; 
                }
            }
    
            $params = array(
                ':NRRETFINANCADQ' => $NRRETFINANCADQ,
                ':IDCONCILIADO'   => $IDCONCILIADO,
                ':IDSITUACAO'     => $IDSITUACAO
            );
            $this->cancelAdjustFactory->changeSituationAdjustments($params);
	    } catch (\Exception $e){
	        $paramsdateAdj = array(
                ':NRRETFINANCADQ'  => $NRRETFINANCADQ
            );
            $paramsMovBanc = array(
                ':NRMOVBANC'  => $NRMOVBANC,
                ':CDREDUCONT' => $CDREDUCONT
            );
	        $this->cancelAdjustFactory->clearBankMoveRet($paramsdateAdj);
            $this->cancelAdjustFactory->movDebBanc($paramsMovBanc);
            $this->cancelAdjustFactory->movCredBanc($paramsMovBanc);
	        $this->cancelAdjustFactory->clearBankMove($paramsMovBanc);
	        throw $e;
	    }
    }
    
    public function getErrors($params) {
        $CDEMPRESA	   = $params[0]["value"];
		$NRLANCCTBTEMP = $params[1]["value"];
		$DSERROS       = [];
		$params = array(
			':CDEMPRESA' 		 => $CDEMPRESA,
			':NRLANCCTBTEMP'      => $NRLANCCTBTEMP
		);
		$error = $this->cancelAdjustFactory->getError($params);
		foreach ($error as $key => $value) {
		    $NRSEQCTBTEMP = $error[$key]["NRSEQCTBTEMP"];
		    $DSERRO = $this->errosPadroes[$error[$key]["CDERROTEMP"]];
		    if(strrpos($DSERRO, "%s") !== false){
		        $DSERRO = str_replace('%s', $NRSEQCTBTEMP, $DSERRO);
                $DSERROS[$key]["DSERRO"] = $DSERRO;
            } else {
                $DSERROS[$key]["DSERRO"] = $DSERRO;
            }
		}
		$this->cancelAdjustFactory->deleteErrors($params);
		return $DSERROS;
	}
	
	public function deleteErrors ($params) {
        $CDEMPRESA   = $params[0]['CDEMPRESA'];
		$NRLANCCTBTEMP = $params[0]['NRLANCCTBTEMP'];

		$paramethers = array(
			':CDEMPRESA'   => $CDEMPRESA,
			':NRLANCCTBTEMP' => $NRLANCCTBTEMP
		);

		$this->cancelAdjustFactory->deleteErrors($paramethers);
	}
    
    public function refreshCashFlow($CDEMPRESA, $CDFILIAL, $DTCREDITO, $CDCLASSFINA, $VRLIQPGTO, $TPG){
        $params["P_CDEMPRESA"] = $CDEMPRESA;
        $params["P_CDFILIAL"] = $CDFILIAL;
        $params["P_DTCREDITO"] = $DTCREDITO;
        $params["P_CDCLASSFINA"] = $CDCLASSFINA;
        if($TPG == 'D'){
            $params["P_VRENTMFLUXO"] = 0;
            $params["P_VRSAIMFLUXO"] = abs($VRLIQPGTO);
        }else{
            $params["P_VRENTMFLUXO"] = abs($VRLIQPGTO);
            $params["P_VRSAIMFLUXO"] = 0;
        }
        $this->cancelAdjustFactory->refreshCashFlow($params);
    }
    
    public function refreshBankBalance($CDREDUCONT, $DTCREDITO, $VRLIQPGTO, $TPG){
        $params["P_CDREDUCONT"] = $CDREDUCONT;
        $params["P_DTCREDITO"]  = $DTCREDITO;
        if($TPG == 'D'){
            $params["P_VRSDOCCORR"]  = abs($VRLIQPGTO)*-1;
        }else{
            $params["P_VRSDOCCORR"]  = abs($VRLIQPGTO); 
        }
        $this->cancelAdjustFactory->refreshBankBalance($params);
    }
    
    public function ctbBcoLanc($CDREDUCONT, $NRLANCCTBTEMP, $IDTIPO){
        $params["P_CDREDUCONT"] = $CDREDUCONT;
        $params["P_NRLANCCTBTEMP"]  = $NRLANCCTBTEMP;
        $params["P_IDTIPO"]  = $IDTIPO;
        $result = $this->cancelAdjustFactory->ctbBcoLanc($params);
        return $result; 
    }

}