<?php

namespace Teknisa\CNC\Services;
use Teknisa\Libs\Util\Environment;
use Teknisa\CNC\Utilities\UtilCNC;
use Teknisa\CNC\Factories\Cnc02800DataInicioIntegracaoFactory;

class Cnc02800DataInicioIntegracaoService {
    protected $factory;
    protected $environment;
    
    public function __construct (Environment $environment, Cnc02800DataInicioIntegracaoFactory $factory) {
        $this->environment = $environment;
        $this->factory = $factory;
    }

    public function findDataOperador($NRORG) {
        return $this->factory->findDataOperador(array(
            ':NRORG' => $this->environment->getNrOrgTrab()
        ));
    }
    
    public function findDataOperadorFilial() {
        return $this->factory->findDataOperadorFilial(array(
            ':NRORG'    => $this->environment->getNrOrgTrab()
        ));
    }
    
    public function saveDataOperador ($row, $isNew) {
        $row[0]["CDOPERULTATU"] = $this->environment->getCdOperador();
        $row[0]["DTULTATU"] = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
        $row[0]["NRORGULTATU"] = $this->environment->getNrOrgTrab();
        if($isNew){
            $row[0]["IDATIVO"]         = "S";
            $row[0]["DTINCLUSAO"]      = $row[0]["DTULTATU"];
            $row[0]["NRORGINCLUSAO"]   = $row[0]["NRORGULTATU"];
            $row[0]["CDOPERINCLUSAO"]  = $row[0]["CDOPERULTATU"];
            $this->factory->insertDataOperador(array(
                ":NRORG"            => $this->environment->getNrOrgTrab(),
                ":IDATIVO"            => $row[0]["IDATIVO"],
                ":DTINCLUSAO"         => $row[0]["DTINCLUSAO"],
                ":NRORGINCLUSAO"      => $row[0]["NRORGINCLUSAO"],
                ":CDOPERINCLUSAO"     => $row[0]["CDOPERINCLUSAO"],
                ":DTINICIOINT"        => $row[0]["DTINICIOINT"],
                ":CDOPERULTATU"       => $row[0]["CDOPERULTATU"],
                ":DTULTATU"           => $row[0]["DTULTATU"],
                ":NRORGULTATU"        => $row[0]["NRORGULTATU"]
            ));
        }else{
            $necessityInclude = $this->verifyExistInclude();
            if($necessityInclude){
                $row[0]["IDATIVO"]         = "S";
                $row[0]["DTINCLUSAO"]      = $row[0]["DTULTATU"];
                $row[0]["NRORGINCLUSAO"]   = $row[0]["NRORGULTATU"];
                $row[0]["CDOPERINCLUSAO"]  = $row[0]["CDOPERULTATU"];
                 $this->factory->updateDataOperadorInclude(array(
                    ":NRORG"            => $this->environment->getNrOrgTrab(),
                    ":DTINICIOINT"        => $row[0]["DTINICIOINT"],
                    ":CDOPERULTATU"       => $row[0]["CDOPERULTATU"],
                    ":DTULTATU"           => $row[0]["DTULTATU"],
                    ":NRORGULTATU"        => $row[0]["NRORGULTATU"],
                    ":IDATIVO"            => $row[0]["IDATIVO"],
                    ":DTINCLUSAO"         => $row[0]["DTINCLUSAO"],
                    ":NRORGINCLUSAO"      => $row[0]["NRORGINCLUSAO"],
                    ":CDOPERINCLUSAO"     => $row[0]["CDOPERINCLUSAO"]
                ));  
            }else{
                 $this->factory->updateDataOperador(array(
                    ":NRORG"            => $this->environment->getNrOrgTrab(),
                    ":DTINICIOINT"        => $row[0]["DTINICIOINT"],
                    ":CDOPERULTATU"       => $row[0]["CDOPERULTATU"],
                    ":DTULTATU"           => $row[0]["DTULTATU"],
                    ":NRORGULTATU"        => $row[0]["NRORGULTATU"]
                ));   
            }
        }
    }
    
    public function saveDataOperadorFilial ($row, $isNew) {
        $row[0]["CDOPERULTATU"] = $this->environment->getCdOperador();
        $row[0]["DTULTATU"] = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
        $row[0]["NRORGULTATU"] = $this->environment->getNrOrgTrab();
        if($isNew){
            $row[0]["IDATIVO"]         = "S";
            $row[0]["DTINCLUSAO"]      = $row[0]["DTULTATU"];
            $row[0]["NRORGINCLUSAO"]   = $row[0]["NRORGULTATU"];
            $row[0]["CDOPERINCLUSAO"]  = $row[0]["CDOPERULTATU"];
            $this->factory->insertDataOperadorFilial(array(
                ":NRORG"              => $this->environment->getNrOrgTrab(),
                ":CDFILIAL"           => $row[0]["CDFILIAL"],
                ":IDATIVO"            => $row[0]["IDATIVO"],
                ":DTINCLUSAO"         => $row[0]["DTINCLUSAO"],
                ":NRORGINCLUSAO"      => $row[0]["NRORGINCLUSAO"],
                ":CDOPERINCLUSAO"     => $row[0]["CDOPERINCLUSAO"],
                ":DTINICIOINT"        => $row[0]["DTINICIOINT"],
                ":CDOPERULTATU"       => $row[0]["CDOPERULTATU"],
                ":DTULTATU"           => $row[0]["DTULTATU"],
                ":NRORGULTATU"        => $row[0]["NRORGULTATU"]
            ));
        }else{
            $necessityInclude = $this->verifyExistIncludeFilial($row[0]);
            if($necessityInclude){
                $row[0]["IDATIVO"]         = "S";
                $row[0]["DTINCLUSAO"]      = $row[0]["DTULTATU"];
                $row[0]["NRORGINCLUSAO"]   = $row[0]["NRORGULTATU"];
                $row[0]["CDOPERINCLUSAO"]  = $row[0]["CDOPERULTATU"];
                 $this->factory->updateDataOperadorIncludeFilial(array(
                    ":NRORG"            => $this->environment->getNrOrgTrab(),
                    ":CDFILIAL"           => $row[0]["CDFILIAL"],
                    ":DTINICIOINT"        => $row[0]["DTINICIOINT"],
                    ":CDOPERULTATU"       => $row[0]["CDOPERULTATU"],
                    ":DTULTATU"           => $row[0]["DTULTATU"],
                    ":NRORGULTATU"        => $row[0]["NRORGULTATU"],
                    ":IDATIVO"            => $row[0]["IDATIVO"],
                    ":DTINCLUSAO"         => $row[0]["DTINCLUSAO"],
                    ":NRORGINCLUSAO"      => $row[0]["NRORGINCLUSAO"],
                    ":CDOPERINCLUSAO"     => $row[0]["CDOPERINCLUSAO"]
                ));  
            }else{
                 $this->factory->updateDataOperadorFilial(array(
                    ":NRORG"              => $this->environment->getNrOrgTrab(),
                    ":CDFILIAL"           => $row[0]["CDFILIAL"],
                    ":DTINICIOINT"        => $row[0]["DTINICIOINT"],
                    ":CDOPERULTATU"       => $row[0]["CDOPERULTATU"],
                    ":DTULTATU"           => $row[0]["DTULTATU"],
                    ":NRORGULTATU"        => $row[0]["NRORGULTATU"]
                ));   
            }
        }
    }
    
    public function verifyExistData($row){
        $result = $this->factory->findData(array(
            ':NRORG' => $this->environment->getNrOrgTrab()
        ));
        
        if(empty($result)){
            return true;
        }else{
            return false;
        }
        
    }
    
    public function verifyExistDataFilial ($row){
        $result = $this->factory->findDataFilial(array(
            ':NRORG'    => $this->environment->getNrOrgTrab(),
            ':CDFILIAL' => $row[0]['CDFILIAL']
        ));
        
        if(empty($result)){
            return true;
        }else{
            return false;
        }
        
    }
    
    public function verifyExistInclude(){
        $result = $this->factory->verifyInclude(array(
            ':NRORG' => $this->environment->getNrOrgTrab()
        ));
        
        if(empty($result[0]["NRORGINCLUSAO"])){
            return true;
        }else{
            return false;
        }
        
    }
    
    public function verifyExistIncludeFilial($row){
        $result = $this->factory->verifyIncludeFilial(array(
            ':NRORG' => $this->environment->getNrOrgTrab(),
            ':CDFILIAL' => $row['CDFILIAL']
        ));
        
        if(empty($result[0]["NRORGINCLUSAO"])){
            return true;
        }else{
            return false;
        }
    }
}
