<?php 

namespace WebServiceCall\Repository\DB;

class Estabelecimento extends ABaseRepo {
    private const ESTABELECIMENTO_TABLE_NAME = 'ESTABELECIMENTO';

    public function insertEstabelecimento($cdEstabelecimento) {
        $nrEstabelec = $this->conn->getNextTableId(self::ESTABELECIMENTO_TABLE_NAME);

        $params = array(
            ':NRESTABELEC'  => $nrEstabelec,
            ':CDESTABELEC'  => $cdEstabelecimento,
            ':NRORG'       => $this->conn->getNRORG(),
        );

        $result = $this->conn->executeQuery(self::INSERT_ESTABELECIMENTO, $params, [], true);
    }
    
    public function getEstabelecimentoByCode($cdEstabelecimento) {
        $params = array(
            ':CDESTABELEC' => $cdEstabelecimento,
        );
        
        $result = $this->conn->fetchAssoc(self::GET_ESTABELECIMENTO_BY_CODE, $params);
        return $result;
    }
    
    public function getNrEstabelecByCode($cdEstabelecimento) {
        $params = array(
            ':CDESTABELEC' => $cdEstabelecimento,
        );
        
        $result = $this->conn->fetchRow(self::GET_NRESTABELEC_BY_CODE, $params);
        return $result[0];
    }
    
    public function getNrEstabelecsByCodes($cdEstabelecimentos) {
        $params = array(
            ':CDESTABELEC' => implode(', ', $cdEstabelecimentos),
        );
        
        $result = $this->conn->fetchAll(self::GET_NRESTABELECS_BY_CODES, $params, [], true);
        return !empty($result) ? $result['NRESTABELEC'] : null;
    }

    public function updateEstabelecFilialParams($nrEstabelec, $cnpj, $filialParams) {
        $params = array (
            ':CDFILIAL'    => $filialParams["CDFILIAL"],
            ':NMESTABELEC' => $filialParams["NMFILIAL"],
            ':CNPJ'        => $cnpj,
            ':CDEMPRESA'   => $filialParams["CDEMPRESA"],
            ':CDINSCESTA'  => $filialParams["CDINSCESTA"],
            ':CDINSCMUNI'  => $filialParams["CDINSCMUNI"],
            ':NRORG'       => $this->conn->getNRORG(),
            ':NRESTABELEC' => $nrEstabelec,
        );

        $this->conn->executeQuery(self::UPDATE_FILIAL_PARAMS_ON_ESTABELECIMENTO, $params);
    }    
    
    private const INSERT_ESTABELECIMENTO = 
        "INSERT INTO ESTABELECIMENTO (
            NRESTABELEC, 
            CDESTABELEC, 
            NRORG,
            IDATIVO
        ) VALUES (
            :NRESTABELEC, 
            TO_CHAR(TO_NUMBER(:CDESTABELEC)), 
            :NRORG, 
            'S'
        )";
        
    private const GET_ESTABELECIMENTO_BY_CODE = 
        "SELECT * 
           FROM ESTABELECIMENTO 
          WHERE TO_NUMBER(CDESTABELEC) = TO_NUMBER(:CDESTABELEC)";
          
          
    private const GET_NRESTABELEC_BY_CODE = 
        "SELECT NRESTABELEC 
           FROM ESTABELECIMENTO 
          WHERE TO_NUMBER(CDESTABELEC) = TO_NUMBER(:CDESTABELEC)";
    
    private const GET_NRESTABELECS_BY_CODES = 
        "SELECT NRESTABELEC
           FROM ESTABELECIMENTO
          WHERE TO_NUMBER(CDESTABELEC) IN (:CDESTABELEC)";
          
    private const UPDATE_FILIAL_PARAMS_ON_ESTABELECIMENTO = "
            UPDATE ESTABELECIMENTO 
               SET CDFILIAL = :CDFILIAL, 
                   NMESTABELEC = :NMESTABELEC, 
                   NRCNPJESTABELEC = :CNPJ, 
                   CDEMPRESA = :CDEMPRESA, 
                   CDINSCESTA = :CDINSCESTA, 
                   CDINSCMUNI = :CDINSCMUNI 
             WHERE NRORG = :NRORG 
               AND NRESTABELEC = :NRESTABELEC
    ";
}
